module AdvisoryLock
  ADVISORY_LOCK = self

  def self.included(base)
    base.extend ClassMethods
  end

  module ClassMethods
    def exclusive(lock_identifier = self.class.name.hash, &block)
      ADVISORY_LOCK.exclusive(lock_identifier, connection, &block)
    end
  end

  module Implementation
    def exclusive(lock_identifier, connection = ActiveRecord::Base.connection, &_block)
      if obtained_lock?(lock_identifier, connection)
        begin
          yield
        ensure
          release_lock(lock_identifier, connection)
        end
      else
        raise "Cannot get lock #{lock_identifier.inspect}"
      end
    end

    private

    def obtained_lock?(lock_identifier, connection)
      connection.select_value("select pg_try_advisory_lock(#{lock_identifier})")
    end

    def release_lock(lock_identifier, connection)
      connection.execute "select pg_advisory_unlock(#{lock_identifier})"
    end
  end

  extend Implementation
end
