module Tracker
  module Registry
    extend self

    def reset!
      @registrations = nil
    end

    private

    def registrations
      @registrations ||= Hash.new { |h, k| h[k] = [] }
    end

    def register_callback(event, &proc)
      STDERR.puts "Starting to track #{event.inspect}"
      callbacks(event) << proc
    end

    def callbacks(event)
      registrations[event]
    end

    public

    def on(event, &proc)
      expect! event => /(insert|delete|update)$/
      register_callback event, &proc
    end

    def track(table, &proc)
      register_callback table, &proc
    end

    def tracks?(table)
      !callbacks(table).empty?
    end

    def publish!(event_name, *args)
      callbacks(event_name).each do |callback|
        callback.call(*args)
      end
    end
  end
end
